@@ -1,7 +1,7 @@
 from autoarray import preprocess
 from autoarray import Mask2D
-from autoarray.dataset.imaging import Imaging
-from autoarray.dataset.interferometer import Interferometer
+from autoarray.dataset.imaging import Imaging, SettingsImaging
+from autoarray.dataset.interferometer import Interferometer, SettingsInterferometer
 from autoarray.mask.mask_2d import Mask2D
 from autoarray.operators.convolver import Convolver
 from autoarray.inversion import pixelizations as pix, regularization as reg
@@ -10,34 +10,26 @@ from autoarray.inversion.inversions import inversion as Inversion, SettingsInver
 from autoarray.inversion.mappers import mapper as Mapper
 from autoarray.operators.transformer import TransformerDFT
 from autoarray.operators.transformer import TransformerNUFFT
-from autoarray.operators.transformer import TransformerNUFFT
 from autoarray.structures.arrays.one_d.array_1d import Array1D
 from autoarray.structures.arrays.two_d.array_2d import Array2D
 from autoarray.structures.arrays.values import ValuesIrregular
 from autoarray.structures.grids.one_d.grid_1d import Grid1D
 from autoarray.structures.grids.two_d.grid_2d import Grid2D
+from autoarray.structures.grids.two_d.grid_2d import Grid2DSparse
 from autoarray.structures.grids.two_d.grid_2d_interpolate import Grid2DInterpolate
 from autoarray.structures.grids.two_d.grid_2d_iterate import Grid2DIterate
 from autoarray.structures.grids.two_d.grid_2d_irregular import Grid2DIrregular
 from autoarray.structures.grids.two_d.grid_2d_irregular import Grid2DIrregularUniform
 from autoarray.structures.grids.two_d.grid_2d_pixelization import Grid2DRectangular
 from autoarray.structures.grids.two_d.grid_2d_pixelization import Grid2DVoronoi
-from autoarray.structures.vector_fields.vector_field_irregular import (
-    VectorField2DIrregular,
-)
+from autoarray.structures.vector_fields.vector_field_irregular import VectorField2DIrregular
 from autoarray.structures.kernel_2d import Kernel2D
 from autoarray.structures.visibilities import Visibilities
 from autogalaxy import util
-from autogalaxy.dataset.imaging import SettingsMaskedImaging
-from autogalaxy.dataset.interferometer import SettingsMaskedInterferometer
 from autogalaxy.galaxy.fit_galaxy import FitGalaxy
 from autogalaxy.galaxy.galaxy import Galaxy, HyperGalaxy, Redshift
 from autogalaxy.galaxy.galaxy_data import GalaxyData
-from autogalaxy.galaxy.galaxy_model import GalaxyModel
 from autogalaxy.hyper import hyper_data
-from autogalaxy.pipeline.setup import SetupLightParametric, SetupSMBH
-from autogalaxy.pipeline.phase.extensions.hyper_phase import HyperPhase
-from autogalaxy.pipeline.pipeline import PipelineDataset
 from autogalaxy.plane.plane import Plane
 from autogalaxy.profiles import (
     point_sources as ps,
@@ -47,11 +39,14 @@ from autogalaxy.profiles import (
 )
 from autogalaxy import convert
 
-from . import aggregator as agg
+from .analysis import aggregator as agg
 from . import plot
-from .dataset.imaging import MaskedImaging, SimulatorImaging
-from .dataset.interferometer import MaskedInterferometer, SimulatorInterferometer
-from .fit.fit import FitImaging, FitInterferometer
+from .dataset.imaging import SimulatorImaging
+from .dataset.interferometer import SimulatorInterferometer
+from .dataset.point_source import PointSourceDataset
+from .dataset.point_source import PointSourceDict
+from .fit.fit_imaging import FitImaging
+from .fit.fit_interferometer import FitInterferometer
 from .fit.fit_point_source import (
     FitPositionsSourceMaxSeparation,
     FitPositionsImage,
@@ -60,33 +55,16 @@ from .fit.fit_point_source import (
 from .lens.settings import SettingsLens
 from .lens.ray_tracing import Tracer
 from .lens.positions_solver import PositionsSolver
-from .pipeline.setup import (
-    SetupPipeline,
-    SetupHyper,
-    SetupSourceParametric,
-    SetupSourceInversion,
-    SetupMassTotal,
-    SetupMassLightDark,
-    SetupSubhalo,
+from .analysis.preloads import Preloads
+from .analysis.analysis import (
+    AnalysisImaging,
+    AnalysisInterferometer,
+    AnalysisPointSource,
 )
-from .pipeline.slam import (
-    SLaMPipelineSourceParametric,
-    SLaMPipelineSourceInversion,
-    SLaMPipelineLightParametric,
-    SLaMPipelineMass,
-    SLaM,
-)
-from .pipeline.phase.settings import SettingsPhaseImaging
-from .pipeline.phase.settings import SettingsPhaseInterferometer
-from .pipeline.phase.settings import SettingsPhasePositions
-from .pipeline.phase.imaging.phase import PhaseImaging
-from .pipeline.phase.interferometer.phase import PhaseInterferometer
-from .pipeline.phase.point_source.phase import PhasePointSource
-from .pipeline.phase.extensions.stochastic_phase import StochasticPhase
-from .pipeline.phase.phase_galaxy import PhaseGalaxy
+from autolens.analysis.setup import SetupHyper
 
 from autoconf import conf
 
 conf.instance.register(__file__)
 
-__version__ = "1.13.0"
+__version__ = "1.15.1"
