@@ -1,54 +1,16 @@
 from autoarray.dataset import imaging
 from autoarray.structures.grids.two_d import grid_2d
 from autoarray.structures import kernel_2d
-from autogalaxy.dataset import imaging as im
 from autolens.lens import ray_tracing
 
 
-class MaskedImaging(imaging.MaskedImaging):
-    def __init__(self, imaging, mask, settings=im.SettingsMaskedImaging()):
-        """
-        The lens dataset is the collection of data (image, noise-map, PSF), a mask, grid, convolver \
-        and other utilities that are used for modeling and fitting an image of a strong lens.
-
-        Whilst the image, noise-map, etc. are loaded in 2D, the lens dataset creates reduced 1D arrays of each \
-        for lens calculations.
-
-        Parameters
-        ----------
-        imaging: im.Imaging
-            The imaging data all in 2D (the image, noise-map, PSF, etc.)
-        mask: msk.Mask2D
-            The 2D mask that is applied to the image.
-        sub_size : int
-            The size of the sub-grid used for each lens SubGrid. E.g. a value of 2 grid each image-pixel on a 2x2 \
-            sub-grid.
-        psf_shape_2d : (int, int)
-            The shape of the PSF used for convolving model image generated using analytic light profiles. A smaller \
-            shape will trim the PSF relative to the input image PSF, giving a faster analysis run-time.
-        positions : [[]]
-            Lists of image-pixel coordinates (arc-seconds) that mappers close to one another in the source-plane(s), \
-            used to speed up the non-linear sampling.
-        pixel_scales_interp : float
-            If `True`, expensive to compute mass profile deflection angles will be computed on a sparse grid and \
-            interpolated to the grid, sub and blurring grids.
-        inversion_pixel_limit : int or None
-            The maximum number of pixels that can be used by an inversion, with the limit placed primarily to speed \
-            up run.
-        """
-
-        super(MaskedImaging, self).__init__(
-            imaging=imaging, mask=mask, settings=settings
-        )
-
-
 class SimulatorImaging(imaging.SimulatorImaging):
     def __init__(
         self,
         exposure_time: float,
         background_sky_level: float = 0.0,
         psf: kernel_2d.Kernel2D = None,
-        renormalize_psf: bool = True,
+        normalize_psf: bool = True,
         read_noise: float = None,
         add_poisson_noise: bool = True,
         noise_if_add_noise_false: float = 0.1,
@@ -65,8 +27,8 @@ class SimulatorImaging(imaging.SimulatorImaging):
             The exposure time of the simulated imaging.
         background_sky_level : float
             The level of the background sky of the simulated imaging.
-        renormalize_psf : bool
-            If `True`, the PSF kernel is renormalized so all values sum to 1.0.
+        normalize_psf : bool
+            If `True`, the PSF kernel is normalized so all values sum to 1.0.
         read_noise : float
             The level of read-noise added to the simulated imaging by drawing from a Gaussian distribution with
             sigma equal to the value `read_noise`.
@@ -83,7 +45,7 @@ class SimulatorImaging(imaging.SimulatorImaging):
             psf=psf,
             exposure_time=exposure_time,
             background_sky_level=background_sky_level,
-            renormalize_psf=renormalize_psf,
+            normalize_psf=normalize_psf,
             read_noise=read_noise,
             add_poisson_noise=add_poisson_noise,
             noise_if_add_noise_false=noise_if_add_noise_false,
@@ -114,11 +76,11 @@ class SimulatorImaging(imaging.SimulatorImaging):
             A seed for random noise_maps generation
         """
 
-        image = tracer.padded_image_from_grid_and_psf_shape(
+        image = tracer.padded_image_2d_from_grid_and_psf_shape(
             grid=grid, psf_shape_2d=self.psf.shape_native
         )
 
-        imaging = self.from_image(image=image.slim_binned, name=name)
+        imaging = self.from_image(image=image.binned, name=name)
 
         return imaging.trimmed_after_convolution_from(
             kernel_shape=self.psf.shape_native
@@ -152,8 +114,8 @@ class SimulatorImaging(imaging.SimulatorImaging):
             sub_size=1,
         )
 
-        deflected_grid = grid - deflections.slim_binned
+        deflected_grid = grid - deflections.binned
 
-        image = sum(map(lambda g: g.image_from_grid(grid=deflected_grid), galaxies))
+        image = sum(map(lambda g: g.image_2d_from_grid(grid=deflected_grid), galaxies))
 
         return self.from_image(image=image, name=name)
