@@ -13,13 +13,21 @@ class PlottingException(Exception):
     pass
 
 
-class PhaseException(Exception):
-    pass
-
-
 class PixelizationException(af.exc.FitException):
     pass
 
 
 class SettingsException(Exception):
     pass
+
+
+class AnalysisException(Exception):
+    pass
+
+
+class PreloadException(Exception):
+    pass
+
+
+class PointSourceExtractionException(Exception):
+    pass
