@@ -3,6 +3,8 @@ import numpy as np
 from autoarray.structures.grids.two_d import grid_2d_irregular
 from autogalaxy.profiles import mass_profiles as mp
 
+from functools import partial
+
 
 class AbstractPositionsSolver:
     def __init__(
@@ -20,11 +22,13 @@ class AbstractPositionsSolver:
         self.distance_from_source_centre = distance_from_source_centre
         self.distance_from_mass_profile_centre = distance_from_mass_profile_centre
 
-    def grid_with_points_below_magnification_threshold_removed(self, lensing_obj, grid):
+    def grid_with_points_below_magnification_threshold_removed(
+        self, lensing_obj, deflections_func, grid
+    ):
 
         magnifications = np.abs(
             lensing_obj.magnification_via_hessian_from_grid(
-                grid=grid, buffer=grid.pixel_scale
+                grid=grid, buffer=grid.pixel_scale, deflections_func=deflections_func
             )
         )
 
@@ -53,7 +57,7 @@ class AbstractPositionsSolver:
         ----------
         lensing_obj : autogalaxy.LensingObject
             An object which has a deflection_from_grid method for performing lensing calculations, for example a
-            `MassProfile`, _Galaxy_, `Plane` or _Tracer_.
+            `MassProfile`, _Galaxy_, `Plane` or `Tracer`.
         grid : autoarray.Grid2DIrregularUniform or ndarray
             A gridd of (y,x) Cartesian coordinates for which their distances to the mass profile centres are computed,
             with points within the threshold removed.
@@ -62,7 +66,9 @@ class AbstractPositionsSolver:
 
             pixel_scales = grid.pixel_scales
 
-            centres = lensing_obj.extract_attribute(cls=mp.MassProfile, name="centre")
+            centres = lensing_obj.extract_attribute(
+                cls=mp.MassProfile, attr_name="centre"
+            )
 
             for centre in centres.in_list:
 
@@ -137,8 +143,9 @@ class AbstractPositionsSolver:
             ),
         )
 
-    def grid_peaks_from(self, lensing_obj, grid, source_plane_coordinate):
-        """Find the 'peaks' of a grid of coordinates, where a peak corresponds to a (y,x) coordinate on the grid which
+    def grid_peaks_from(self, deflections_func, grid, source_plane_coordinate):
+        """
+        Find the 'peaks' of a grid of coordinates, where a peak corresponds to a (y,x) coordinate on the grid which
         traces closer to the input (y,x) source-plane coordinate than any of its 8 adjacent neighbors. This is
         performed by:
 
@@ -155,7 +162,7 @@ class AbstractPositionsSolver:
         ----------
         lensing_obj : autogalaxy.LensingObject
             An object which has a deflection_from_grid method for performing lensing calculations, for example a
-            `MassProfile`, _Galaxy_, `Plane` or _Tracer_.
+            `MassProfile`, _Galaxy_, `Plane` or `Tracer`.
         grid : autoarray.Grid2DIrregularUniform or ndarray
             A grid of (y,x) Cartesian coordinates for which the 'peak' values that trace closer to the source than
             their neighbors are found.
@@ -163,8 +170,10 @@ class AbstractPositionsSolver:
             The (y,x) coordinate in the source-plane pixels that the distance of traced grid coordinates are computed
             for.
         """
-        deflections = lensing_obj.deflections_from_grid(grid=grid)
+
+        deflections = deflections_func(grid=grid)
         source_plane_grid = grid.grid_from_deflection_grid(deflection_grid=deflections)
+
         source_plane_distances = source_plane_grid.distances_from_coordinate(
             coordinate=source_plane_coordinate
         )
@@ -185,7 +194,7 @@ class AbstractPositionsSolver:
         )
 
     def grid_within_distance_of_source_plane_centre(
-        self, lensing_obj, source_plane_coordinate, grid, distance
+        self, deflection_func, source_plane_coordinate, grid, distance
     ):
         """
         For an input grid of (y,x) coordinates, remove all coordinates that do not trace within a threshold distance
@@ -204,7 +213,7 @@ class AbstractPositionsSolver:
             ----------
             lensing_obj : autogalaxy.LensingObject
                 An object which has a deflection_from_grid method for performing lensing calculations, for example a
-                `MassProfile`, _Galaxy_, `Plane` or _Tracer_.
+                `MassProfile`, _Galaxy_, `Plane` or `Tracer`.
             grid : autoarray.Grid2DIrregularUniform or ndarray
                 A grid of (y,x) Cartesian coordinates for which the 'peak' values that trace closer to the source than
                 their neighbors are found.
@@ -217,7 +226,7 @@ class AbstractPositionsSolver:
         if distance is None:
             return grid
 
-        deflections = lensing_obj.deflections_from_grid(grid=grid)
+        deflections = deflection_func(grid=grid)
         source_plane_grid = grid.grid_from_deflection_grid(deflection_grid=deflections)
         source_plane_distances = source_plane_grid.distances_from_coordinate(
             coordinate=source_plane_coordinate
@@ -245,8 +254,8 @@ class PositionsSolver(AbstractPositionsSolver):
         distance_from_source_centre=None,
         distance_from_mass_profile_centre=None,
     ):
-        """Given a `LensingObject` (e.g. a _MassProfile, `Galaxy`, `Plane` or _Tracer_) this class uses their
-        deflections_from_grid method to determine the (y,x) coordinates the multiple-images appear given a (y,x)
+        """Given a `LensingObject` (e.g. a _MassProfile, `Galaxy`, `Plane` or `Tracer`) this class uses their
+        deflections_2d_from_grid method to determine the (y,x) coordinates the multiple-images appear given a (y,x)
         source-centre coordinate in the source-plane.
 
         This is performed as follows:
@@ -269,7 +278,7 @@ class PositionsSolver(AbstractPositionsSolver):
             8 neighbors. Depending on how the `PositionFinder` is being used these can be removed.
         """
 
-        super(PositionsSolver, self).__init__(
+        super().__init__(
             use_upscaling=use_upscaling,
             upscale_factor=upscale_factor,
             magnification_threshold=magnification_threshold,
@@ -277,11 +286,11 @@ class PositionsSolver(AbstractPositionsSolver):
             distance_from_mass_profile_centre=distance_from_mass_profile_centre,
         )
 
-        self.grid = grid.slim_binned
+        self.grid = grid.binned
         self.pixel_scale_precision = pixel_scale_precision
 
     def refined_coordinates_from_coordinate(
-        self, coordinate, pixel_scale, lensing_obj, source_plane_coordinate
+        self, deflections_func, coordinate, pixel_scale, source_plane_coordinate
     ):
         """For an input (y,x) coordinate, determine a set of refined coordinates that are computed by locating peak
         pixels on a higher resolution grid around that pixel.
@@ -298,7 +307,7 @@ class PositionsSolver(AbstractPositionsSolver):
             upscale > 1, the pixel_scales are reduced to pixel_scale / upscale_factor.
         lensing_obj : autogalaxy.LensingObject
             An object which has a deflection_from_grid method for performing lensing calculations, for example a
-            `MassProfile`, _Galaxy_, `Plane` or _Tracer_.
+            `MassProfile`, _Galaxy_, `Plane` or `Tracer`.
         source_plane_coordinate : (float, float)
             The (y,x) coordinate in the source-plane pixels that the distance of traced grid coordinates are computed
             for.
@@ -312,7 +321,7 @@ class PositionsSolver(AbstractPositionsSolver):
         )
 
         grid = self.grid_peaks_from(
-            lensing_obj=lensing_obj,
+            deflections_func=deflections_func,
             grid=grid,
             source_plane_coordinate=source_plane_coordinate,
         )
@@ -322,12 +331,21 @@ class PositionsSolver(AbstractPositionsSolver):
         else:
             return [tuple(coordinate) for coordinate in grid]
 
-    def solve(self, lensing_obj, source_plane_coordinate):
+    def solve(self, lensing_obj, source_plane_coordinate, upper_plane_index=None):
+
+        if upper_plane_index is None:
+            deflections_func = lensing_obj.deflections_2d_from_grid
+        else:
+            deflections_func = partial(
+                lensing_obj.deflections_between_planes_from_grid,
+                plane_i=0,
+                plane_j=upper_plane_index,
+            )
 
         coordinates_list = self.grid_peaks_from(
-            lensing_obj=lensing_obj,
             grid=self.grid,
             source_plane_coordinate=source_plane_coordinate,
+            deflections_func=deflections_func,
         )
 
         coordinates_list = self.grid_with_coordinates_from_mass_profile_centre_removed(
@@ -335,7 +353,9 @@ class PositionsSolver(AbstractPositionsSolver):
         )
 
         coordinates_list = self.grid_with_points_below_magnification_threshold_removed(
-            lensing_obj=lensing_obj, grid=coordinates_list
+            lensing_obj=lensing_obj,
+            deflections_func=deflections_func,
+            grid=coordinates_list,
         )
 
         if not self.use_upscaling:
@@ -351,9 +371,9 @@ class PositionsSolver(AbstractPositionsSolver):
             for coordinate in coordinates_list:
 
                 refined_coordinates = self.refined_coordinates_from_coordinate(
+                    deflections_func=deflections_func,
                     coordinate=coordinate,
                     pixel_scale=pixel_scale,
-                    lensing_obj=lensing_obj,
                     source_plane_coordinate=source_plane_coordinate,
                 )
 
@@ -369,7 +389,7 @@ class PositionsSolver(AbstractPositionsSolver):
             coordinates_list = refined_coordinates_list
 
         coordinates_list = self.grid_within_distance_of_source_plane_centre(
-            lensing_obj=lensing_obj,
+            deflection_func=deflections_func,
             grid=grid_2d_irregular.Grid2DIrregularUniform(
                 grid=coordinates_list, pixel_scales=(pixel_scale, pixel_scale)
             ),
@@ -378,7 +398,9 @@ class PositionsSolver(AbstractPositionsSolver):
         )
 
         coordinates_list = self.grid_with_points_below_magnification_threshold_removed(
-            lensing_obj=lensing_obj, grid=coordinates_list
+            lensing_obj=lensing_obj,
+            deflections_func=deflections_func,
+            grid=coordinates_list,
         )
 
         return grid_2d_irregular.Grid2DIrregular(grid=coordinates_list)
