@@ -13,11 +13,13 @@ from autogalaxy.galaxy import galaxy as g
 from autogalaxy.plane import plane as pl
 from autogalaxy.util import cosmology_util
 from autogalaxy.util import plane_util
+from autoarray import preloads as pload
 
 
 class AbstractTracer(lensing.LensingObject, ABC):
     def __init__(self, planes, cosmology):
-        """Ray-tracer for a lens system with any number of planes.
+        """
+        Ray-tracer for a lens system with any number of planes.
 
         The redshift of these planes are specified by the redshits of the galaxies; there is a unique plane redshift \
         for every unique galaxy redshift (galaxies with identical redshifts are put in the same plane).
@@ -123,7 +125,7 @@ class AbstractTracer(lensing.LensingObject, ABC):
     def planes_with_mass_profile(self):
         return list(filter(lambda plane: plane.has_mass_profile, self.planes))
 
-    def extract_attribute(self, cls, name):
+    def extract_attribute(self, cls, attr_name):
         """
         Returns an attribute of a class in the tracer as a `ValueIrregular` or `Grid2DIrregular` object.
 
@@ -138,7 +140,7 @@ class AbstractTracer(lensing.LensingObject, ABC):
         If the image plane has has two galaxies with two mass profiles and the source plane another galaxy with a
         mass profile, the following:
 
-        `tracer.extract_attribute(cls=MassProfile, name="centres")`
+        `tracer.extract_attribute(cls=MassProfile, name="centre")`
 
         would return:
 
@@ -155,7 +157,7 @@ class AbstractTracer(lensing.LensingObject, ABC):
                 return None
 
         attributes = [
-            extract(value, name)
+            extract(value, attr_name)
             for galaxy in self.galaxies
             for value in galaxy.__dict__.values()
             if isinstance(value, cls)
@@ -168,7 +170,7 @@ class AbstractTracer(lensing.LensingObject, ABC):
         elif isinstance(attributes[0], tuple):
             return grid_2d_irregular.Grid2DIrregular(grid=attributes)
 
-    def extract_attributes_of_planes(self, cls, name, filter_nones=False):
+    def extract_attributes_of_planes(self, cls, attr_name, filter_nones=False):
         """
         Returns an attribute of a class in the tracer as a list of `ValueIrregular` or `Grid2DIrregular` objects, where
         the indexes of the list correspond to the tracer's planes.
@@ -202,18 +204,19 @@ class AbstractTracer(lensing.LensingObject, ABC):
         if filter_nones:
 
             return [
-                plane.extract_attribute(cls=cls, name=name)
+                plane.extract_attribute(cls=cls, attr_name=attr_name)
                 for plane in self.planes
-                if plane.extract_attribute(cls=cls, name=name) is not None
+                if plane.extract_attribute(cls=cls, attr_name=attr_name) is not None
             ]
 
         else:
 
             return [
-                plane.extract_attribute(cls=cls, name=name) for plane in self.planes
+                plane.extract_attribute(cls=cls, attr_name=attr_name)
+                for plane in self.planes
             ]
 
-    def extract_attributes_of_galaxies(self, cls, name, filter_nones=False):
+    def extract_attributes_of_galaxies(self, cls, attr_name, filter_nones=False):
         """
         Returns an attribute of a class in the tracer as a list of `ValueIrregular` or `Grid2DIrregular` objects, where
         the indexes of the list correspond to the tracer's galaxies. If a plane has multiple galaxies they are split
@@ -258,17 +261,52 @@ class AbstractTracer(lensing.LensingObject, ABC):
         if filter_nones:
 
             return [
-                galaxy.extract_attribute(cls=cls, name=name)
+                galaxy.extract_attribute(cls=cls, attr_name=attr_name)
                 for galaxy in self.galaxies
-                if galaxy.extract_attribute(cls=cls, name=name) is not None
+                if galaxy.extract_attribute(cls=cls, attr_name=attr_name) is not None
             ]
 
         else:
 
             return [
-                galaxy.extract_attribute(cls=cls, name=name) for galaxy in self.galaxies
+                galaxy.extract_attribute(cls=cls, attr_name=attr_name)
+                for galaxy in self.galaxies
             ]
 
+    def extract_profile(self, profile_name):
+        """
+        Returns a `LightProfile`, `MassProfile` or `PointSource` from the `Tracer` using the name of that component.
+
+        For example, if a tracer has two galaxies, `lens` and `source` with `LightProfile`'s name `light_0` and
+        `light_1`, the following:
+
+        `tracer.extract_profile(profile_name="light_1")`
+
+        Would return the `LightProfile` of the source plane.
+        """
+        for galaxy in self.galaxies:
+            try:
+                return galaxy.__dict__[profile_name]
+            except KeyError:
+                pass
+
+    def extract_plane_index_of_profile(self, profile_name):
+        """
+        Returns the plane index of a  LightProfile`, `MassProfile` or `PointSource` from the `Tracer` using the name
+        of that component.
+
+        For example, if a tracer has two galaxies, `lens` and `source` with `LightProfile`'s name `light_0` and
+        `light_1`, the following:
+
+        `tracer.extract_profile(profile_name="light_1")`
+
+        Would return `plane_index=1` given the profile is in the source plane.
+        """
+        for plane_index, plane in enumerate(self.planes):
+            for galaxy in plane.galaxies:
+                if profile_name in galaxy.__dict__:
+                    return plane_index
+
     @property
     def mass_profiles(self):
         return [
@@ -325,7 +363,7 @@ class AbstractTracer(lensing.LensingObject, ABC):
 
 
 class AbstractTracerLensing(AbstractTracer, ABC):
-    @grid_decorators.grid_like_to_structure_list
+    @grid_decorators.grid_2d_to_structure_list
     def traced_grids_of_planes_from_grid(self, grid, plane_index_limit=None):
 
         traced_grids = []
@@ -358,22 +396,22 @@ class AbstractTracerLensing(AbstractTracer, ABC):
                 if plane_index == plane_index_limit:
                     return traced_grids
 
-            traced_deflections.append(plane.deflections_from_grid(grid=scaled_grid))
+            traced_deflections.append(plane.deflections_2d_from_grid(grid=scaled_grid))
 
         return traced_grids
 
-    @grid_decorators.grid_like_to_structure
+    @grid_decorators.grid_2d_to_structure
     def deflections_between_planes_from_grid(self, grid, plane_i=0, plane_j=-1):
 
         traced_grids_of_planes = self.traced_grids_of_planes_from_grid(grid=grid)
 
         return traced_grids_of_planes[plane_i] - traced_grids_of_planes[plane_j]
 
-    @grid_decorators.grid_like_to_structure
-    def image_from_grid(self, grid):
+    @grid_decorators.grid_2d_to_structure
+    def image_2d_from_grid(self, grid):
         return sum(self.images_of_planes_from_grid(grid=grid))
 
-    @grid_decorators.grid_like_to_structure_list
+    @grid_decorators.grid_2d_to_structure_list
     def images_of_planes_from_grid(self, grid):
 
         traced_grids_of_planes = self.traced_grids_of_planes_from_grid(
@@ -381,7 +419,7 @@ class AbstractTracerLensing(AbstractTracer, ABC):
         )
 
         images_of_planes = [
-            self.planes[plane_index].image_from_grid(
+            self.planes[plane_index].image_2d_from_grid(
                 grid=traced_grids_of_planes[plane_index]
             )
             for plane_index in range(len(traced_grids_of_planes))
@@ -395,29 +433,29 @@ class AbstractTracerLensing(AbstractTracer, ABC):
 
         return images_of_planes
 
-    def padded_image_from_grid_and_psf_shape(self, grid, psf_shape_2d):
+    def padded_image_2d_from_grid_and_psf_shape(self, grid, psf_shape_2d):
 
         padded_grid = grid.padded_grid_from_kernel_shape(
             kernel_shape_native=psf_shape_2d
         )
 
-        return self.image_from_grid(grid=padded_grid)
+        return self.image_2d_from_grid(grid=padded_grid)
 
-    @grid_decorators.grid_like_to_structure
-    def convergence_from_grid(self, grid):
-        return sum([plane.convergence_from_grid(grid=grid) for plane in self.planes])
+    @grid_decorators.grid_2d_to_structure
+    def convergence_2d_from_grid(self, grid):
+        return sum([plane.convergence_2d_from_grid(grid=grid) for plane in self.planes])
 
-    @grid_decorators.grid_like_to_structure
-    def potential_from_grid(self, grid):
-        return sum([plane.potential_from_grid(grid=grid) for plane in self.planes])
+    @grid_decorators.grid_2d_to_structure
+    def potential_2d_from_grid(self, grid):
+        return sum([plane.potential_2d_from_grid(grid=grid) for plane in self.planes])
 
-    @grid_decorators.grid_like_to_structure
-    def deflections_from_grid(self, grid):
+    @grid_decorators.grid_2d_to_structure
+    def deflections_2d_from_grid(self, grid):
         return self.deflections_between_planes_from_grid(grid=grid)
 
-    @grid_decorators.grid_like_to_structure
+    @grid_decorators.grid_2d_to_structure
     def deflections_of_planes_summed_from_grid(self, grid):
-        return sum([plane.deflections_from_grid(grid=grid) for plane in self.planes])
+        return sum([plane.deflections_2d_from_grid(grid=grid) for plane in self.planes])
 
     def grid_at_redshift_from_grid_and_redshift(self, grid, redshift):
         """For an input grid of (y,x) arc-second image-plane coordinates, ray-trace the coordinates to any redshift in \
@@ -492,7 +530,7 @@ class AbstractTracerLensing(AbstractTracer, ABC):
 
 
 class AbstractTracerData(AbstractTracerLensing, ABC):
-    def blurred_image_from_grid_and_psf(self, grid, psf, blurring_grid):
+    def blurred_image_2d_from_grid_and_psf(self, grid, psf, blurring_grid):
         """Extract the 1D image and 1D blurring image of every plane and blur each with the \
         PSF using a psf (see imaging.convolution).
 
@@ -507,12 +545,12 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         if not self.has_light_profile:
             return np.zeros(shape=grid.shape_slim)
 
-        image = self.image_from_grid(grid=grid)
+        image = self.image_2d_from_grid(grid=grid)
 
-        blurring_image = self.image_from_grid(grid=blurring_grid)
+        blurring_image = self.image_2d_from_grid(grid=blurring_grid)
 
         return psf.convolved_array_from_array_and_mask(
-            array=image.native_binned + blurring_image.native_binned, mask=grid.mask
+            array=image.binned.native + blurring_image.binned.native, mask=grid.mask
         )
 
     def blurred_images_of_planes_from_grid_and_psf(self, grid, psf, blurring_grid):
@@ -532,7 +570,7 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
             grid=blurring_grid
         )
         return [
-            plane.blurred_image_from_grid_and_psf(
+            plane.blurred_image_2d_from_grid_and_psf(
                 grid=traced_grids_of_planes[plane_index],
                 psf=psf,
                 blurring_grid=traced_blurring_grids_of_planes[plane_index],
@@ -540,7 +578,7 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
             for (plane_index, plane) in enumerate(self.planes)
         ]
 
-    def blurred_image_from_grid_and_convolver(self, grid, convolver, blurring_grid):
+    def blurred_image_2d_from_grid_and_convolver(self, grid, convolver, blurring_grid):
         """Extract the 1D image and 1D blurring image of every plane and blur each with the \
         PSF using a convolver (see imaging.convolution).
 
@@ -555,13 +593,11 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         if not self.has_light_profile:
             return np.zeros(shape=grid.shape_slim)
 
-        image = self.image_from_grid(grid=grid)
+        image = self.image_2d_from_grid(grid=grid)
 
-        blurring_image = self.image_from_grid(grid=blurring_grid)
+        blurring_image = self.image_2d_from_grid(grid=blurring_grid)
 
-        return convolver.convolved_image_from_image_and_blurring_image(
-            image=image, blurring_image=blurring_image
-        )
+        return convolver.convolve_image(image=image, blurring_image=blurring_image)
 
     def blurred_images_of_planes_from_grid_and_convolver(
         self, grid, convolver, blurring_grid
@@ -583,7 +619,7 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         )
 
         return [
-            plane.blurred_image_from_grid_and_convolver(
+            plane.blurred_image_2d_from_grid_and_convolver(
                 grid=traced_grids_of_planes[plane_index],
                 convolver=convolver,
                 blurring_grid=traced_blurring_grids_of_planes[plane_index],
@@ -591,13 +627,13 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
             for (plane_index, plane) in enumerate(self.planes)
         ]
 
-    def unmasked_blurred_image_from_grid_and_psf(self, grid, psf):
+    def unmasked_blurred_image_2d_from_grid_and_psf(self, grid, psf):
 
         padded_grid = grid.padded_grid_from_kernel_shape(
             kernel_shape_native=psf.shape_native
         )
 
-        padded_image = self.image_from_grid(grid=padded_grid)
+        padded_image = self.image_2d_from_grid(grid=padded_grid)
 
         return padded_grid.mask.unmasked_blurred_array_from_padded_array_psf_and_image_shape(
             padded_array=padded_image, psf=psf, image_shape=grid.mask.shape
@@ -614,7 +650,7 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         unmasked_blurred_images_of_planes = []
 
         for plane, traced_padded_grid in zip(self.planes, traced_padded_grids):
-            padded_image_1d = plane.image_from_grid(grid=traced_padded_grid)
+            padded_image_1d = plane.image_2d_from_grid(grid=traced_padded_grid)
 
             unmasked_blurred_array_2d = padded_grid.mask.unmasked_blurred_array_from_padded_array_psf_and_image_shape(
                 padded_array=padded_image_1d, psf=psf, image_shape=grid.mask.shape
@@ -663,7 +699,7 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         if not self.has_light_profile:
             return np.zeros(shape=transformer.uv_wavelengths.shape[0])
 
-        image = self.image_from_grid(grid=grid)
+        image = self.image_2d_from_grid(grid=grid)
 
         return transformer.visibilities_from_image(image=image)
 
@@ -692,11 +728,14 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         return sparse_image_plane_grids_of_planes
 
     def traced_sparse_grids_of_planes_from_grid(
-        self, grid, settings_pixelization=pix.SettingsPixelization()
+        self,
+        grid,
+        settings_pixelization=pix.SettingsPixelization(),
+        preloads=pload.Preloads(),
     ):
 
         if (
-            settings_pixelization.preload_sparse_grids_of_planes is None
+            preloads.sparse_grids_of_planes is None
             or settings_pixelization.is_stochastic
         ):
 
@@ -706,9 +745,7 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
 
         else:
 
-            sparse_image_plane_grids_of_planes = (
-                settings_pixelization.preload_sparse_grids_of_planes
-            )
+            sparse_image_plane_grids_of_planes = preloads.sparse_grids_of_planes
 
         traced_sparse_grids_of_planes = []
 
@@ -728,7 +765,10 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
             return traced_sparse_grids_of_planes, sparse_image_plane_grids_of_planes[0]
 
     def mappers_of_planes_from_grid(
-        self, grid, settings_pixelization=pix.SettingsPixelization()
+        self,
+        grid,
+        settings_pixelization=pix.SettingsPixelization(),
+        preloads=pload.Preloads(),
     ):
 
         mappers_of_planes = []
@@ -736,7 +776,7 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         traced_grids_of_planes = self.traced_grids_of_planes_from_grid(grid=grid)
 
         traced_sparse_grids_of_planes, sparse_image_plane_grid = self.traced_sparse_grids_of_planes_from_grid(
-            grid=grid, settings_pixelization=settings_pixelization
+            grid=grid, settings_pixelization=settings_pixelization, preloads=preloads
         )
 
         for (plane_index, plane) in enumerate(self.planes):
@@ -762,11 +802,20 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         convolver,
         settings_pixelization=pix.SettingsPixelization(),
         settings_inversion=inv.SettingsInversion(),
+        preloads=pload.Preloads(),
     ):
 
-        mappers_of_planes = self.mappers_of_planes_from_grid(
-            grid=grid, settings_pixelization=settings_pixelization
-        )
+        if preloads.mapper is None:
+
+            mappers_of_planes = self.mappers_of_planes_from_grid(
+                grid=grid,
+                settings_pixelization=settings_pixelization,
+                preloads=preloads,
+            )
+
+        else:
+
+            mappers_of_planes = [preloads.mapper]
 
         return inv.InversionImagingMatrix.from_data_mapper_and_regularization(
             image=image,
@@ -775,6 +824,7 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
             mapper=mappers_of_planes[-1],
             regularization=self.regularizations_of_planes[-1],
             settings=settings_inversion,
+            preloads=preloads,
         )
 
     def inversion_interferometer_from_grid_and_data(
@@ -785,9 +835,10 @@ class AbstractTracerData(AbstractTracerLensing, ABC):
         transformer,
         settings_pixelization=pix.SettingsPixelization(),
         settings_inversion=inv.SettingsInversion(),
+        preloads=pload.Preloads(),
     ):
         mappers_of_planes = self.mappers_of_planes_from_grid(
-            grid=grid, settings_pixelization=settings_pixelization
+            grid=grid, settings_pixelization=settings_pixelization, preloads=preloads
         )
 
         return inv.AbstractInversionInterferometer.from_data_mapper_and_regularization(
