@@ -44,8 +44,6 @@ extensions = [
     "sphinx.ext.napoleon",
     "sphinx.ext.extlinks",
     "numpydoc",
-    "nbsphinx",
-    "nbsphinx_link",
 ]
 
 ## Generate autodoc stubs with summaries from code
@@ -61,8 +59,6 @@ numpydoc_show_class_members = False
 numpydoc_show_inherited_class_members = False
 numpydoc_class_members_toctree = False
 
-nbsphinx_allow_errors = True
-
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ["_templates"]
 
