@@ -33,7 +33,7 @@ The mass-profile and source light profile in this example have fixed centre (0.0
 lensing geometries.
 """
 
-mass_profile_model = af.PriorModel(al.mp.EllipticalIsothermal)
+mass_profile_model = af.Model(al.mp.EllIsothermal)
 mass_profile_model.centre.centre_0 = 0.0
 mass_profile_model.centre.centre_1 = 0.0
 mass_profile_model.elliptical_comps.elliptical_comps_0 = af.UniformPrior(
@@ -61,20 +61,20 @@ for i in range(iters):
     lensed source.
     """
 
-    exponential_light_profile = al.lp.EllipticalExponential(
+    exponential_light_profile = al.lp.EllExponential(
         centre=(0.0, 0.0),
         elliptical_comps=(0.2, 0.0),
         intensity=0.05,
         effective_radius=0.2,
     )
 
-    """Setup the lens, source and _Tracer_."""
+    """Setup the lens, source and `Tracer`."""
 
     lens_galaxy = al.Galaxy(redshift=0.5, mass=mass_profile)
     source_galaxy = al.Galaxy(redshift=1.0, light=exponential_light_profile)
     tracer = al.Tracer.from_galaxies(galaxies=[lens_galaxy, source_galaxy])
 
-    """Solve for the positions via the _Tracer_."""
+    """Solve for the positions via the `Tracer`."""
 
     positions = solver.solve(
         lensing_obj=tracer,
@@ -86,7 +86,7 @@ for i in range(iters):
     visuals_2d = aplt.Visuals2D(positions=positions)
 
     tracer_plotter = aplt.TracerPlotter(tracer=tracer, grid=grid, visuals_2d=visuals_2d)
-    tracer_plotter.figures(image=True)
+    tracer_plotter.figures_2d(image=True)
 
     """Save the `Tracer` and `Positions` so they can be used for testing other `PositionsSolver` settings."""
 
