@@ -16,8 +16,10 @@ class MockClass:
 @pytest.fixture(name="label_config")
 def make_label_config():
 
+    print(directory, "config")
+
     config = conf.Config(
-        path.join(directory, "config"), path.join(directory, "default")
+        path.join(directory, "config"), output_path=path.join(directory, "output")
     )
 
     return config["notation"]["label"]
@@ -32,6 +34,12 @@ class TestLabel:
         assert label_config["label"]["gamma"] == r"\gamma"
         assert label_config["label"]["contribution_factor"] == r"\omega0"
 
+    # def test_subscript(self, label_config):
+    #     assert label_config["subscript"].family(al.lp.EllLightProfile) == "l"
+    #
+    # def test_inheritance(self, label_config):
+    #     assert label_config["subscript"].family(al.lp.EllGaussian) == "l"
+
     def test_exception(self, label_config):
         with pytest.raises(KeyError):
             label_config["subscript"].family(MockClass)
